==> grpc-protobuf-nano-1.13.1-sources.jar: io/grpc/protobuf/nano/NanoProtoInputStream.java <==
/*
 * Copyright 2014 The gRPC Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.grpc.protobuf.nano;

import com.google.protobuf.nano.CodedOutputByteBufferNano;
import com.google.protobuf.nano.MessageNano;
import io.grpc.KnownLength;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.annotation.Nullable;

/**
 * An {@link InputStream} backed by a nano proto.
 */
class NanoProtoInputStream extends InputStream implements KnownLength {

  // NanoProtoInputStream is first initialized with a *message*. *partial* is initially null.
  // Once there has been a read operation on this stream, *message* is serialized to *partial* and
  // set to null.
  @Nullable private MessageNano message;
  @Nullable private ByteArrayInputStream partial;

  public NanoProtoInputStream(MessageNano message) {
    this.message = message;
  }

  private void toPartial() {
    if (message != null) {
      partial = new ByteArrayInputStream(MessageNano.toByteArray(message));
      message = null;
    }
  }

  @Override
  public int read() throws IOException {
    toPartial();
    if (partial != null) {
      return partial.read();
    }
    return -1;
  }

  @Override
  public int read(byte[] b, int off, int len) throws IOException {
    if (message != null) {
      int size = message.getSerializedSize();
      if (size == 0) {
        message = null;
        partial = null;
        return -1;
      }
      if (len >= size) {
        // This is the only case that is zero-copy.
        CodedOutputByteBufferNano output = CodedOutputByteBufferNano.newInstance(b, off, size);
        message.writeTo(output);
        output.checkNoSpaceLeft();

        message = null;
        partial = null;
        return size;
      }

      toPartial();
    }
    if (partial != null) {
      return partial.read(b, off, len);
    }
    return -1;
  }

  @Override
  public int available() throws IOException {
    if (message != null) {
      return message.getSerializedSize();
    } else if (partial != null) {
      return partial.available();
    }
    return 0;
  }
}

==> grpc-protobuf-nano-1.17.1-sources.jar: io/grpc/protobuf/nano/NanoProtoInputStream.java <==
/*
 * Copyright 2014 The gRPC Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.grpc.protobuf.nano;

import com.google.protobuf.nano.CodedOutputByteBufferNano;
import com.google.protobuf.nano.MessageNano;
import io.grpc.KnownLength;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.annotation.Nullable;

/**
 * An {@link InputStream} backed by a nano proto.
 */
final class NanoProtoInputStream extends InputStream implements KnownLength {

  // NanoProtoInputStream is first initialized with a *message*. *partial* is initially null.
  // Once there has been a read operation on this stream, *message* is serialized to *partial* and
  // set to null.
  @Nullable private MessageNano message;
  @Nullable private ByteArrayInputStream partial;

  NanoProtoInputStream(MessageNano message) {
    this.message = message;
  }

  private void toPartial() {
    if (message != null) {
      partial = new ByteArrayInputStream(MessageNano.toByteArray(message));
      message = null;
    }
  }

  @Override
  public int read() throws IOException {
    toPartial();
    if (partial != null) {
      return partial.read();
    }
    return -1;
  }

  @Override
  public int read(byte[] b, int off, int len) throws IOException {
    if (message != null) {
      int size = message.getSerializedSize();
      if (size == 0) {
        message = null;
        partial = null;
        return -1;
      }
      if (len >= size) {
        // This is the only case that is zero-copy.
        CodedOutputByteBufferNano output = CodedOutputByteBufferNano.newInstance(b, off, size);
        message.writeTo(output);
        output.checkNoSpaceLeft();

        message = null;
        partial = null;
        return size;
      }

      toPartial();
    }
    if (partial != null) {
      return partial.read(b, off, len);
    }
    return -1;
  }

  @Override
  public int available() {
    if (message != null) {
      return message.getSerializedSize();
    } else if (partial != null) {
      return partial.available();
    }
    return 0;
  }
}
